/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="ExtTrigDataHV"> ********
Author: Rahkola, R
$Id: ExtTrigData.h,v 1.4 2002/04/18 19:27:58 rrahkola Exp $
********* </lalVerbatim> ********/
#ifndef _EXTTRIGDATA_H
#define _EXTTRIGDATA_H

/* Desired sampling rate of the GW channels */
#ifdef DEBUG_EXTTRIG
#define GW_SAMPLE 2048
#else
#define GW_SAMPLE 16384
#endif

/* LAL functions */
#include <lal/LALRCSID.h>
#include <lal/LALDatatypes.h>
#include <lal/DetectorSite.h>
#include <lal/TimeDelay.h>

#ifdef __cplusplus
extern "C" {
#endif

NRCSID( EXTTRIGDATAH, "$Id: ExtTrigData.h,v 1.4 2002/04/18 19:27:58 rrahkola Exp $");

/********** <lalErrTable file="ExtTrigDataHErrTab"> ******************/
#define EXTTRIGDATAH_ENULL 1
#define EXTTRIGDATAH_ENNUL 2
#define EXTTRIGDATAH_ENSEG 3
#define EXTTRIGDATAH_ENPTS 4
#define EXTTRIGDATAH_EALOC 5
#define EXTTRIGDATAH_EDUMY 6

#define EXTTRIGDATAH_MSGENULL "exttrig:Null pointer"
#define EXTTRIGDATAH_MSGENNUL "exttrig:Non-null pointer"
#define EXTTRIGDATAH_MSGENSEG "exttrig:Number of data segments is zero or less"
#define EXTTRIGDATAH_MSGENPTS "exttrig:Number of points in data segment is zero or less"
#define EXTTRIGDATAH_MSGEALOC "exttrig:Memory allocation error"
#define EXTTRIGDATAH_MSGEDUMY "exttrig:Done executing dummy code"

/************************************ </lalErrTable> */


/* 
 * The user-defined structure which holds the input parameters for our
 * search.
 */
typedef struct tagETInitParams 
{
  /* Containers for the search parameters (to be output to table) */
  /* The GPS start times of the GRB events */
  LIGOTimeGPS             grbStart;
  /* Direction of the GRB event */
  SkyPosition             sourceDir;
  /* Window/Segment length to cross-correlate (in counts)*/
  UINT4                   segLength;
  /* Number of 'off'-source segments to cross-correlate*/
  UINT4                   numOffSegments;
  /* Time interval between beginning of 'on'-source segment and beginning
   * of GRB event (in sec.)
   */
  INT4                    onTimeShift;
  /* Hard lowerlimit for interval between beginning of GRB event and beginning
   * of 'off'-source segment (in sec.)
   */
  UINT4                   offTimeShiftLow;
  /* Hard upperlimit for interval between beginning of GRB event and beginning
   * of 'off'-source segment (in sec.)
   */
  UINT4                   offTimeShiftHigh;
  /* Mask containing the following parameters:                   */
  /* (Reserve 2^0 place for on-source, off-source flag)          */
  /* flag whether to use data quality information (2^1) or not (0) */
  UINT4                   initMask;
}
ETInitParams;


/*
 * The user-defined structure which holds the results of the search
 */
typedef struct tagETResult
{
  /* Containers for calculated results */
  /* Variance of the two cross-correlated time series */
  REAL4                   variance1;
  REAL4                   variance2;
  /* Cross-correlation units & values */
  /* :NOTE: we may want to change this to a vector quantity */
  LALUnit                 xCorrUnit;
  REAL4Sequence          *xCorrValue;
  /* expected delay between events at the IFOs; (+) if chan1 is first, 
   * (-) if chan2 is first
   */
  REAL4Sequence          *xCorrLag;
  /* Actual intervals between beginning of GRB event and 
   * beginning of 'on'- or 'off'-source segment (in seconds)
   */
  REAL4Sequence          *xCorrDelay;
  /* number of 'bad' seconds during 'on'-source segment, as determined by data
   * quality information
   */
  REAL4                   badSeconds;
  /* whether the result applies to an 'on'-source or 'off'-source segment */
  BOOLEAN                 onSource;
} 
ETResult;


/*
 * The user-defined structure which holds the data to be processed.
 */
typedef struct tagETDataSegment 
{
  /* Containers for the timeseries data received from the datacondAPI */
  REAL4TimeSeries         *tsChan1;
  REAL4TimeSeries         *tsChan2;
  COMPLEX8FrequencySeries *filter;
  /* Container for data quality information received from the datacondAPI */
  //REAL4TimeSeries         *dq;
  /* Reference to the ETSegment ID which defines this ETDataSegment */
  UINT4                    segmentID;
} 
ETDataSegment;

/*
 * A vector which identifies the length of the data to be processed.
 */
typedef struct tagETDataSegmentVector 
{
  UINT4                   length;
  UINT4                   filled;
  UINT4                   segsToBeAnalyzed;
  UINT4                   segsAnalyzed;
  ETDataSegment          *data;
} 
ETDataSegmentVector;

void
LALCreateETDataSegmentVector (
			      LALStatus              *status,
			      ETDataSegmentVector   **vector,
			      ETInitParams           *params
			      );

void
LALDestroyETDataSegmentVector (
			       LALStatus             *status,
			       ETDataSegmentVector  **vector
			       );

/*
 * Information required for identifying an off-source segment.
 */
typedef struct tagETSegment
{
  /* The starting GPS time */
  LIGOTimeGPS       startGPS;
  /* The ending GPS time */
  LIGOTimeGPS       endGPS;
  /* The (randomly-selected) delay time for two IFOs */
  REAL8             delay;
  /* The flag whether or not this segment has been selected */
  BOOLEAN           selected;
} ETSegment;

/*
 * The list of segment start times & durations corresponding to the available
 * off-source segments.
 */
typedef struct tagETSegList
{
  UINT4             length;
  UINT4             filled;
  ETSegment        *data;
} ETSegList;

void
LALCreateETSegList (
		    LALStatus     *status,
		    ETSegList    **list,
		    ETInitParams  *params );

void
LALDestroyETSegList (
		     LALStatus  *status,
		     ETSegList **list   );

#ifdef __cplusplus
}
#endif

#endif /* _EXTTRIGDATA_H */
